"use strict";
const utils_1 = require('./utils');
/**
 * Provides logging interface
 */
class LoggingInterface {
    constructor(logRecordFactory, { tags = [], location = '' }, saveFunc) {
        this._saveFunc = saveFunc;
        this._tags = tags;
        this._location = location;
        this._logRecordFactory = logRecordFactory;
    }
    get tags() {
        return this._tags.slice();
    }
    get location() {
        return this._location;
    }
    /**
     * Get new log record
     * @param message Text message that should be logged
     * @returns Returns instance of log record, that can be modified before processing
     */
    l(message) {
        return this._logRecordFactory.create(this.tags, this.location, message, this._saveFunc);
    }
    /**
     * Creates new Log record and process it
     * @param message Text message that describes log record
     * @param data Data attached to the log record
     * @param loc Location, where this log done
     * @param tags List of tags attached for this specific log record
     */
    log(message, data = null, loc = null, tags = []) {
        this.l(message).data(data).loc(loc).tag(tags).log();
    }
    /**
     * Creates new Log record with Emergency tag and process it
     * @param message Text message that describes log record
     * @param data Data attached to the log record
     * @param loc Location, where this log done
     * @param tags List of tags attached for this specific log record
     */
    emerg(message, data = null, loc = null, tags = []) {
        this.log(message, data, loc, utils_1.mergeTags(tags, [utils_1.LOG_EMERGENCY]));
    }
    /**
     * Creates new Log record with Alert tag and process it
     * @param message Text message that describes log record
     * @param data Data attached to the log record
     * @param loc Location, where this log done
     * @param tags List of tags attached for this specific log record
     */
    alert(message, data = null, loc = null, tags = []) {
        this.log(message, data, loc, utils_1.mergeTags(tags, [utils_1.LOG_ALERT]));
    }
    /**
     * Creates new Log record with Critical tag and process it
     * @param message Text message that describes log record
     * @param data Data attached to the log record
     * @param loc Location, where this log done
     * @param tags List of tags attached for this specific log record
     */
    crit(message, data = null, loc = null, tags = []) {
        this.log(message, data, loc, utils_1.mergeTags(tags, [utils_1.LOG_CRITICAL]));
    }
    /**
     * Creates new Log record with Error tag and process it
     * @param message Text message that describes log record
     * @param data Data attached to the log record
     * @param loc Location, where this log done
     * @param tags List of tags attached for this specific log record
     */
    error(message, data = null, loc = null, tags = []) {
        this.log(message, data, loc, utils_1.mergeTags(tags, [utils_1.LOG_ERROR]));
    }
    /**
     * Creates new Log record with Warning tag and process it
     * @param message Text message that describes log record
     * @param data Data attached to the log record
     * @param loc Location, where this log done
     * @param tags List of tags attached for this specific log record
     */
    warning(message, data = null, loc = null, tags = []) {
        this.log(message, data, loc, utils_1.mergeTags(tags, [utils_1.LOG_WARNING]));
    }
    /**
     * Creates new Log record with Notice tag and process it
     * @param message Text message that describes log record
     * @param data Data attached to the log record
     * @param loc Location, where this log done
     * @param tags List of tags attached for this specific log record
     */
    notice(message, data = null, loc = null, tags = []) {
        this.log(message, data, loc, utils_1.mergeTags(tags, [utils_1.LOG_NOTICE]));
    }
    /**
     * Creates new Log record with Info tag and process it
     * @param message Text message that describes log record
     * @param data Data attached to the log record
     * @param loc Location, where this log done
     * @param tags List of tags attached for this specific log record
     */
    info(message, data = null, loc = null, tags = []) {
        this.log(message, data, loc, utils_1.mergeTags(tags, [utils_1.LOG_INFORMATIONAL]));
    }
    /**
     * Creates new Log record with Debug tag and process it
     * @param message Text message that describes log record
     * @param data Data attached to the log record
     * @param loc Location, where this log done
     * @param tags List of tags attached for this specific log record
     */
    debug(message, data = null, loc = null, tags = []) {
        this.log(message, data, loc, utils_1.mergeTags(tags, [utils_1.LOG_DEBUG]));
    }
    /**
     * Serialize error to object
     */
    serializeError(error) {
        return utils_1.serializeError(error);
    }
    /**
     * Get new logging interface that inherits tag and location
     * @param loc Location that will be added to location of the current logging interface
     * @param tags List of tags that will be merged with tags of the current logging interface
     */
    getInterface(loc, tags) {
        return new LoggingInterface(this._logRecordFactory, {
            tags: utils_1.mergeTags(this._tags.slice(), tags),
            location: utils_1.joinLocation(this._location, loc)
        }, this._saveFunc);
    }
}
exports.LoggingInterface = LoggingInterface;
